// Copyright 2021 Esri.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import UIKit
import ArcGIS

class ViewController: UIViewController {
    @IBOutlet weak var mapView: AGSMapView!

    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupMap()

        setupUI()

        setupGraphicsOverlay()

    }

    // MARK: - Map

    private func setupMap() {

        let map = AGSMap(
            item: AGSPortalItem(
                portal: AGSPortal.arcGISOnline(withLoginRequired: false),
                itemID: "5a030a31e42841a89914bd7c5ecf4d8f"
            )
        )
        mapView.map = map

    }

    // MARK: - UI

    private func setupUI() {
        navigationItem.titleView = UIProgressView()

        navigationController?.isToolbarHidden = false
        let items = [
            UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil),
            UIBarButtonItem(title: "Download Map Area", style: .plain, target: self, action: #selector(userSelectedDownloadOfflineMap)),
            UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil)
        ]
        setToolbarItems(items, animated: true)

    }

    // MARK: - Graphics

    private let graphicsOverlay = AGSGraphicsOverlay()

    private func setupGraphicsOverlay() {
        graphicsOverlay.renderer = AGSSimpleRenderer(
            symbol: AGSSimpleFillSymbol(
                style: .solid,
                color: .clear,
                outline: AGSSimpleLineSymbol(
                    style: .solid,
                    color: .red,
                    width: 3
                )
            )
        )
        mapView.graphicsOverlays.add(graphicsOverlay)
    }

    private func addGraphic(for offlineArea: AGSGeometry) {
        let graphic = AGSGraphic(geometry: offlineArea, symbol: nil)
        graphicsOverlay.graphics.add(graphic)
    }

    // MARK: - Offline Map Job

    @objc
    func userSelectedDownloadOfflineMap(_ sender: UIBarButtonItem) {

        guard let offlineArea = mapView.visibleArea else { return }

        sender.isEnabled = false

        addGraphic(for: offlineArea)

        mapView.setViewpointGeometry(offlineArea, padding: 25)

        downloadOfflineMap(with: offlineArea, at: temporaryDirectoryURL)

    }

    private var temporaryDirectoryURL: URL {
        FileManager.default.temporaryDirectory
            .appendingPathComponent(ProcessInfo().globallyUniqueString)
    }

    private var offlineMapTask: AGSOfflineMapTask?

    private var offlineMapJob: AGSGenerateOfflineMapJob?

    private func downloadOfflineMap(with offlineArea: AGSGeometry, at downloadDirectory: URL) {

        guard let map = mapView.map else { return }
        offlineMapTask = AGSOfflineMapTask(onlineMap: map)

        offlineMapTask?.defaultGenerateOfflineMapParameters(withAreaOfInterest: offlineArea) { [weak self] parameters, error in
            guard let self = self else { return }
            guard let offlineMapTask = self.offlineMapTask else { return }

            if let parameters = parameters {
                parameters.updateMode = .noUpdates
                parameters.esriVectorTilesDownloadOption = .useReducedFontsService
                let job = offlineMapTask.generateOfflineMapJob(with: parameters, downloadDirectory: downloadDirectory)
                (self.navigationItem.titleView as! UIProgressView).observedProgress = job.progress
                var n = 0
                job.start(statusHandler: { _ in
                    while n < job.messages.count {
                        print("Job message \(n): \(job.messages[n].message)")
                        n += 1
                    }
                }, completion: { [weak self] result, error in
                    guard let self = self else { return }
                    if let result = result {
                        self.mapView.map = result.offlineMap
                    } else if let error = error {
                        print("Error downloading the offline map: \(error)")
                        return
                    }
                })
                self.offlineMapJob = job
            } else if let error = error {
                print("Error fetching default parameters for the area of interest: \(error.localizedDescription)")
            }

        }

    }

}

